%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Bousi-Prolog interactive system launcher

:- module(bousi, [
		main/0,
		exit/0                %
   ]).

:- use_module(bplShell).
:- use_module(foreign).
:- use_module(test).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- set_prolog_flag(double_quotes, codes).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main predicate for launching the Bousi-Prolog system
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% main
%
%     Initial predicate of the Bousi-Prolog system. This predicate
%     loads the foreign library, shows a welcome message and
%     initializes and launches the Bousi-Prolog command-line shell.
%

main :-
	foreign:load_foreign_extension,
	flags:set_bpl_flag(continue('yes')),
	welcome_message, % WARNING: Uncomment on final deployment
	bplShell:start_bpl_shell.

	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Predicate for continuing the execution of the Bousi-Prolog system
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% exit
%
%     Continues with Bousi-Prolog execution after a bk command.
%
	
exit :-
	flags:set_bpl_flag(continue('yes')),
	bplShell:bpl_shell_loop.
	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Miscellaneous predicates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% welcome_message
%
%     Shows Bousi-Prolog initial welcome message.
%

welcome_message :-
	nl,
	write('                                                    Universidad de'), nl,
	write('|O)               |D)                            Castilla - La Mancha'), nl,
	write('|O)(O)\\U(S)|I| ~~ || |R (O) |L (O) (G|.     (Version 3.0 ~~ May 2018)'), nl,
	write('----------------------------------------------------------------------------'), nl,
	write('Welcome to Bousi~Prolog, a fuzzy logic programming system created by'), nl,
	write('Juan Gallardo-Casero and Pascual Julian-Iranzo. Fernando Saenz-Perez'), nl,
	write('(UCM) contributed to this version. This software is for research and'), nl, 
	write('educational purposes only and it is distributed with NO WARRANTY.'), nl,
	write('Please visit our website for the latest news on Bousi~Prolog:'), nl,
	write('http://dectau.uclm.es/bousi'), nl,
	write('----------------------------------------------------------------------------'), nl,
	nl,
	write('----------------------------------------------------------------------------'), nl,
	write('--            Enter \'hp\' to get help on the available commands            --'), nl,
	write('----------------------------------------------------------------------------'), nl,
	nl.

